@@ -4,29 +4,35 @@ import logging
 import multiprocessing
 from concurrent.futures.process import ProcessPoolExecutor
 from enum import Enum
-from typing import Dict, List, Optional, Set, Tuple
+from typing import Dict, List, Optional, Set, Tuple, Union
 
 from chia.consensus.block_body_validation import validate_block_body
 from chia.consensus.block_header_validation import validate_finished_header_block, validate_unfinished_header_block
 from chia.consensus.block_record import BlockRecord
 from chia.consensus.blockchain_interface import BlockchainInterface
 from chia.consensus.constants import ConsensusConstants
+from chia.consensus.cost_calculator import NPCResult
 from chia.consensus.difficulty_adjustment import get_next_sub_slot_iters_and_difficulty
 from chia.consensus.find_fork_point import find_fork_point_in_chain
 from chia.consensus.full_block_to_block_record import block_to_block_record
 from chia.consensus.multiprocess_validation import PreValidationResult, pre_validate_blocks_multiprocessing
 from chia.full_node.block_store import BlockStore
 from chia.full_node.coin_store import CoinStore
+from chia.full_node.mempool_check_conditions import get_name_puzzle_conditions
+from chia.types.blockchain_format.coin import Coin
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.types.blockchain_format.sub_epoch_summary import SubEpochSummary
 from chia.types.blockchain_format.vdf import VDFInfo
+from chia.types.coin_record import CoinRecord
 from chia.types.end_of_slot_bundle import EndOfSubSlotBundle
 from chia.types.full_block import FullBlock
+from chia.types.generator_types import BlockGenerator, GeneratorArg
 from chia.types.header_block import HeaderBlock
 from chia.types.unfinished_block import UnfinishedBlock
 from chia.types.unfinished_header_block import UnfinishedHeaderBlock
 from chia.types.weight_proof import SubEpochChallengeSegment
 from chia.util.errors import Err
+from chia.util.generator_tools import get_block_header, block_removals_and_additions
 from chia.util.ints import uint16, uint32, uint64, uint128
 from chia.util.streamable import recurse_jsonify
 
@@ -148,16 +154,6 @@ class Blockchain(BlockchainInterface):
         assert block is not None
         return block
 
-    def is_child_of_peak(self, block: UnfinishedBlock) -> bool:
-        """
-        True if the block is the direct ancestor of the peak
-        """
-        peak = self.get_peak()
-        if peak is None:
-            return False
-
-        return block.prev_header_hash == peak.header_hash
-
     async def get_full_block(self, header_hash: bytes32) -> Optional[FullBlock]:
         return await self.block_store.get_full_block(header_hash)
 
@@ -189,6 +185,7 @@ class Blockchain(BlockchainInterface):
         if not genesis and (self.block_record(block.prev_header_hash).height + 1) != block.height:
             return ReceiveBlockResult.INVALID_BLOCK, Err.INVALID_HEIGHT, None
 
+        npc_result: Optional[NPCResult] = None
         if pre_validation_result is None:
             if block.height == 0:
                 prev_b: Optional[BlockRecord] = None
@@ -197,10 +194,29 @@ class Blockchain(BlockchainInterface):
             sub_slot_iters, difficulty = get_next_sub_slot_iters_and_difficulty(
                 self.constants, len(block.finished_sub_slots) > 0, prev_b, self
             )
+
+            if block.is_transaction_block():
+                if block.transactions_generator is not None:
+                    try:
+                        block_generator: Optional[BlockGenerator] = await self.get_block_generator(block)
+                    except ValueError:
+                        return ReceiveBlockResult.INVALID_BLOCK, Err.GENERATOR_REF_HAS_NO_GENERATOR, None
+                    assert block_generator is not None and block.transactions_info is not None
+                    npc_result = get_name_puzzle_conditions(
+                        block_generator, min(self.constants.MAX_BLOCK_COST_CLVM, block.transactions_info.cost), False
+                    )
+                    removals, additions = block_removals_and_additions(block, npc_result.npc_list)
+                else:
+                    removals, additions = [], list(block.get_included_reward_coins())
+                header_block = get_block_header(block, additions, removals)
+            else:
+                npc_result = None
+                header_block = get_block_header(block, [], [])
+
             required_iters, error = validate_finished_header_block(
                 self.constants,
                 self,
-                block.get_block_header(),
+                header_block,
                 False,
                 difficulty,
                 sub_slot_iters,
@@ -209,6 +225,7 @@ class Blockchain(BlockchainInterface):
             if error is not None:
                 return ReceiveBlockResult.INVALID_BLOCK, error.code, None
         else:
+            npc_result = pre_validation_result.npc_result
             required_iters = pre_validation_result.required_iters
             assert pre_validation_result.error is None
         assert required_iters is not None
@@ -220,8 +237,9 @@ class Blockchain(BlockchainInterface):
             self.get_peak(),
             block,
             block.height,
-            pre_validation_result.cost_result if pre_validation_result is not None else None,
+            npc_result,
             fork_point_with_peak,
+            self.get_block_generator,
         )
         if error_code is not None:
             return ReceiveBlockResult.INVALID_BLOCK, error_code, None
@@ -239,7 +257,7 @@ class Blockchain(BlockchainInterface):
                 await self.block_store.db_wrapper.begin_transaction()
                 await self.block_store.add_full_block(block, block_record)
                 fork_height, peak_height, records = await self._reconsider_peak(
-                    block_record, genesis, fork_point_with_peak
+                    block_record, genesis, fork_point_with_peak, npc_result
                 )
                 await self.block_store.db_wrapper.commit_transaction()
                 self.add_block_record(block_record)
@@ -260,7 +278,11 @@ class Blockchain(BlockchainInterface):
             return ReceiveBlockResult.ADDED_AS_ORPHAN, None, None
 
     async def _reconsider_peak(
-        self, block_record: BlockRecord, genesis: bool, fork_point_with_peak: Optional[uint32]
+        self,
+        block_record: BlockRecord,
+        genesis: bool,
+        fork_point_with_peak: Optional[uint32],
+        npc_result: Optional[NPCResult],
     ) -> Tuple[Optional[uint32], Optional[uint32], List[BlockRecord]]:
         """
         When a new block is added, this is called, to check if the new block is the new peak of the chain.
@@ -276,7 +298,12 @@ class Blockchain(BlockchainInterface):
 
                 # Begins a transaction, because we want to ensure that the coin store and block store are only updated
                 # in sync.
-                await self.coin_store.new_block(block)
+                if npc_result is not None:
+                    removals, additions = block_removals_and_additions(block, npc_result.npc_list)
+                else:
+                    removals = []
+                    additions = list(block.get_included_reward_coins())
+                await self.coin_store.new_block(block, additions, removals)
                 await self.block_store.set_peak(block.header_hash)
                 return uint32(0), uint32(0), [block_record]
             return None, None, []
@@ -290,8 +317,6 @@ class Blockchain(BlockchainInterface):
             else:
                 fork_height = find_fork_point_in_chain(self, block_record, peak)
 
-            # Begins a transaction, because we want to ensure that the coin store and block store are only updated
-            # in sync.
             # Rollback to fork
             await self.coin_store.rollback_to_block(fork_height)
             # Rollback sub_epoch_summaries
@@ -303,11 +328,6 @@ class Blockchain(BlockchainInterface):
                 log.info(f"delete ses at height {height}")
                 del self.__sub_epoch_summaries[height]
 
-            if len(heights_to_delete) > 0:
-                # remove segments from prev fork
-                log.info(f"remove segments for se above {fork_height}")
-                await self.block_store.delete_sub_epoch_challenge_segments(uint32(fork_height))
-
             # Collect all blocks from fork point to new peak
             blocks_to_add: List[Tuple[FullBlock, BlockRecord]] = []
             curr = block_record.header_hash
@@ -322,11 +342,16 @@ class Blockchain(BlockchainInterface):
                     # Doing a full reorg, starting at height 0
                     break
                 curr = fetched_block_record.prev_hash
+
             records_to_add = []
             for fetched_full_block, fetched_block_record in reversed(blocks_to_add):
                 records_to_add.append(fetched_block_record)
                 if fetched_block_record.is_transaction_block:
-                    await self.coin_store.new_block(fetched_full_block)
+                    if fetched_block_record.header_hash == block_record.header_hash:
+                        removals, additions = await self.get_removals_and_additions(fetched_full_block, npc_result)
+                    else:
+                        removals, additions = await self.get_removals_and_additions(fetched_full_block, None)
+                    await self.coin_store.new_block(fetched_full_block, additions, removals)
 
             # Changes the peak to be the new peak
             await self.block_store.set_peak(block_record.header_hash)
@@ -335,6 +360,22 @@ class Blockchain(BlockchainInterface):
         # This is not a heavier block than the heaviest we have seen, so we don't change the coin set
         return None, None, []
 
+    async def get_removals_and_additions(
+        self, block: FullBlock, npc_result: Optional[NPCResult] = None
+    ) -> Tuple[List[bytes32], List[Coin]]:
+        if block.is_transaction_block():
+            if block.transactions_generator is not None:
+                if npc_result is None:
+                    block_generator: Optional[BlockGenerator] = await self.get_block_generator(block)
+                    assert block_generator is not None
+                    npc_result = get_name_puzzle_conditions(block_generator, self.constants.MAX_BLOCK_COST_CLVM, False)
+                removals, additions = block_removals_and_additions(block, npc_result.npc_list)
+                return removals, additions
+            else:
+                return [], list(block.get_included_reward_coins())
+        else:
+            return [], []
+
     def get_next_difficulty(self, header_hash: bytes32, new_slot: bool) -> uint64:
         assert self.contains_block(header_hash)
         curr = self.block_record(header_hash)
@@ -408,9 +449,10 @@ class Blockchain(BlockchainInterface):
         if peak is None:
             return []
         recent_rc: List[Tuple[bytes32, uint128]] = []
-        curr = self.try_block_record(peak.prev_hash)
+        curr: Optional[BlockRecord] = peak
         while curr is not None and len(recent_rc) < 2 * self.constants.MAX_SUB_SLOT_BLOCKS:
-            recent_rc.append((curr.reward_infusion_new_challenge, curr.total_iters))
+            if curr != peak:
+                recent_rc.append((curr.reward_infusion_new_challenge, curr.total_iters))
             if curr.first_in_sub_slot:
                 assert curr.finished_reward_slot_hashes is not None
                 sub_slot_total_iters = curr.ip_sub_slot_total_iters(self.constants)
@@ -462,6 +504,18 @@ class Blockchain(BlockchainInterface):
             else self.block_record(block.prev_header_hash).height
         )
 
+        npc_result = None
+        if block.transactions_generator is not None:
+            assert block.transactions_info is not None
+            try:
+                block_generator: Optional[BlockGenerator] = await self.get_block_generator(block)
+            except ValueError:
+                return PreValidationResult(uint16(Err.GENERATOR_REF_HAS_NO_GENERATOR.value), None, None)
+            if block_generator is None:
+                return PreValidationResult(uint16(Err.GENERATOR_REF_HAS_NO_GENERATOR.value), None, None)
+            npc_result = get_name_puzzle_conditions(
+                block_generator, min(self.constants.MAX_BLOCK_COST_CLVM, block.transactions_info.cost), False
+            )
         error_code, cost_result = await validate_block_body(
             self.constants,
             self,
@@ -470,7 +524,9 @@ class Blockchain(BlockchainInterface):
             self.get_peak(),
             block,
             uint32(prev_height + 1),
+            npc_result,
             None,
+            self.get_block_generator,
         )
 
         if error_code is not None:
@@ -479,10 +535,18 @@ class Blockchain(BlockchainInterface):
         return PreValidationResult(None, required_iters, cost_result)
 
     async def pre_validate_blocks_multiprocessing(
-        self, blocks: List[FullBlock], validate_transactions: bool = True
+        self, blocks: List[FullBlock], npc_results: Dict[uint32, NPCResult], batch_size: int = 4
     ) -> Optional[List[PreValidationResult]]:
         return await pre_validate_blocks_multiprocessing(
-            self.constants, self.constants_json, self, blocks, self.pool, validate_transactions, True
+            self.constants,
+            self.constants_json,
+            self,
+            blocks,
+            self.pool,
+            True,
+            npc_results,
+            self.get_block_generator,
+            batch_size,
         )
 
     def contains_block(self, header_hash: bytes32) -> bool:
@@ -568,7 +632,26 @@ class Blockchain(BlockchainInterface):
         return await self.block_store.get_block_records_in_range(start, stop)
 
     async def get_header_blocks_in_range(self, start: int, stop: int) -> Dict[bytes32, HeaderBlock]:
-        return await self.block_store.get_header_blocks_in_range(start, stop)
+        hashes = []
+        for height in range(start, stop + 1):
+            if self.contains_height(uint32(height)):
+                header_hash: bytes32 = self.height_to_hash(uint32(height))
+                hashes.append(header_hash)
+
+        blocks: List[FullBlock] = await self.block_store.get_blocks_by_hash(hashes)
+        header_blocks: Dict[bytes32, HeaderBlock] = {}
+
+        for block in blocks:
+            if self.height_to_hash(block.height) != block.header_hash:
+                raise ValueError(f"Block at {block.header_hash} is no longer in the blockchain (it's in a fork)")
+            additions: List[CoinRecord] = await self.coin_store.get_coins_added_at_height(block.height)
+            removed: List[CoinRecord] = await self.coin_store.get_coins_removed_at_height(block.height)
+            header = get_block_header(
+                block, [record.coin for record in additions], [record.coin.name() for record in removed]
+            )
+            header_blocks[header.header_hash] = header
+
+        return header_blocks
 
     async def get_header_block_by_height(self, height: int, header_hash: bytes32) -> Optional[HeaderBlock]:
         header_dict: Dict[bytes32, HeaderBlock] = await self.get_header_blocks_in_range(height, height)
@@ -607,23 +690,17 @@ class Blockchain(BlockchainInterface):
             self.__heights_in_cache[block_record.height] = set()
         self.__heights_in_cache[block_record.height].add(block_record.header_hash)
 
-    async def get_header_block(self, header_hash: bytes32) -> Optional[HeaderBlock]:
-        block = await self.block_store.get_full_block(header_hash)
-        if block is None:
-            return None
-        return block.get_block_header()
-
     async def persist_sub_epoch_challenge_segments(
-        self, sub_epoch_summary_height: uint32, segments: List[SubEpochChallengeSegment]
+        self, ses_block_hash: bytes32, segments: List[SubEpochChallengeSegment]
     ):
-        return await self.block_store.persist_sub_epoch_challenge_segments(sub_epoch_summary_height, segments)
+        return await self.block_store.persist_sub_epoch_challenge_segments(ses_block_hash, segments)
 
     async def get_sub_epoch_challenge_segments(
         self,
-        sub_epoch_summary_height: uint32,
+        ses_block_hash: bytes32,
     ) -> Optional[List[SubEpochChallengeSegment]]:
         segments: Optional[List[SubEpochChallengeSegment]] = await self.block_store.get_sub_epoch_challenge_segments(
-            sub_epoch_summary_height
+            ses_block_hash
         )
         if segments is None:
             return None
@@ -639,3 +716,79 @@ class Blockchain(BlockchainInterface):
             self._seen_compact_proofs.clear()
         self._seen_compact_proofs.add(pot_tuple)
         return False
+
+    async def get_block_generator(
+        self, block: Union[FullBlock, UnfinishedBlock], additional_blocks=None
+    ) -> Optional[BlockGenerator]:
+        if additional_blocks is None:
+            additional_blocks = {}
+        ref_list = block.transactions_generator_ref_list
+        if block.transactions_generator is None:
+            assert len(ref_list) == 0
+            return None
+        if len(ref_list) == 0:
+            return BlockGenerator(block.transactions_generator, [])
+
+        result: List[GeneratorArg] = []
+        previous_block_hash = block.prev_header_hash
+        if (
+            self.try_block_record(previous_block_hash)
+            and self.height_to_hash(self.block_record(previous_block_hash).height) == previous_block_hash
+        ):
+            # We are not in a reorg, no need to look up alternate header hashes (we can get them from height_to_hash)
+            for ref_height in block.transactions_generator_ref_list:
+                header_hash = self.height_to_hash(ref_height)
+                ref_block = await self.get_full_block(header_hash)
+                assert ref_block is not None
+                if ref_block.transactions_generator is None:
+                    raise ValueError(Err.GENERATOR_REF_HAS_NO_GENERATOR)
+                result.append(GeneratorArg(ref_block.height, ref_block.transactions_generator))
+        else:
+            # First tries to find the blocks in additional_blocks
+            reorg_chain: Dict[uint32, FullBlock] = {}
+            curr: Union[FullBlock, UnfinishedBlock] = block
+            additional_height_dict = {}
+            while curr.prev_header_hash in additional_blocks:
+                prev: FullBlock = additional_blocks[curr.prev_header_hash]
+                additional_height_dict[prev.height] = prev
+                if isinstance(curr, FullBlock):
+                    assert curr.height == prev.height + 1
+                reorg_chain[prev.height] = prev
+                curr = prev
+
+            peak: Optional[BlockRecord] = self.get_peak()
+            if self.contains_block(curr.prev_header_hash) and peak is not None:
+                # Then we look up blocks up to fork point one at a time, backtracking
+                previous_block_hash = curr.prev_header_hash
+                prev_block_record = await self.block_store.get_block_record(previous_block_hash)
+                prev_block = await self.block_store.get_full_block(previous_block_hash)
+                assert prev_block is not None
+                assert prev_block_record is not None
+                fork = find_fork_point_in_chain(self, peak, prev_block_record)
+                curr_2: Optional[FullBlock] = prev_block
+                assert curr_2 is not None and isinstance(curr_2, FullBlock)
+                reorg_chain[curr_2.height] = curr_2
+                while curr_2.height > fork and curr_2.height > 0:
+                    curr_2 = await self.block_store.get_full_block(curr_2.prev_header_hash)
+                    assert curr_2 is not None
+                    reorg_chain[curr_2.height] = curr_2
+
+            for ref_height in block.transactions_generator_ref_list:
+                if ref_height in reorg_chain:
+                    ref_block = reorg_chain[ref_height]
+                    assert ref_block is not None
+                    if ref_block.transactions_generator is None:
+                        raise ValueError(Err.GENERATOR_REF_HAS_NO_GENERATOR)
+                    result.append(GeneratorArg(ref_block.height, ref_block.transactions_generator))
+                else:
+                    if ref_height in additional_height_dict:
+                        ref_block = additional_height_dict[ref_height]
+                    else:
+                        header_hash = self.height_to_hash(ref_height)
+                        ref_block = await self.get_full_block(header_hash)
+                    assert ref_block is not None
+                    if ref_block.transactions_generator is None:
+                        raise ValueError(Err.GENERATOR_REF_HAS_NO_GENERATOR)
+                    result.append(GeneratorArg(ref_block.height, ref_block.transactions_generator))
+        assert len(result) == len(ref_list)
+        return BlockGenerator(block.transactions_generator, result)
