@@ -15,7 +15,7 @@ from typing import Any, Dict, List, Optional, TextIO, Tuple, cast
 
 from websockets import ConnectionClosedOK, WebSocketException, WebSocketServerProtocol, serve
 
-from chia.cmds.init import chia_init
+from chia.cmds.init_funcs import chia_init
 from chia.daemon.windows_signal import kill
 from chia.server.server import ssl_context_for_root, ssl_context_for_server
 from chia.ssl.create_ssl import get_mozzila_ca_crt
@@ -66,6 +66,7 @@ class PlotState(str, Enum):
     SUBMITTED = "SUBMITTED"
     RUNNING = "RUNNING"
     ERROR = "ERROR"
+    DELETED = "DELETED"
     FINISHED = "FINISHED"
 
 
@@ -101,7 +102,7 @@ else:
         return service_name
 
 
-async def ping():
+async def ping() -> Dict:
     response = {"success": True, "value": "pong"}
     return response
 
@@ -152,11 +153,12 @@ class WebSocketServer:
             except Exception as e:
                 self.log.error(f"Error while canceling task.{e} {task}")
 
-    async def stop(self):
+    async def stop(self) -> Dict:
         self.shut_down = True
         self.cancel_task_safe(self.ping_job)
         await self.exit()
-        self.websocket_server.close()
+        if self.websocket_server is not None:
+            self.websocket_server.close()
         return {"success": True}
 
     async def safe_handle(self, websocket: WebSocketServerProtocol, path: str):
@@ -212,7 +214,7 @@ class WebSocketServer:
                     after_removal.append(connection)
             self.connections[service_name] = after_removal
 
-    async def ping_task(self):
+    async def ping_task(self) -> None:
         restart = True
         await asyncio.sleep(30)
         for remote_address, service_name in self.remote_address_map.items():
@@ -288,11 +290,11 @@ class WebSocketServer:
         full_response = format_response(message, response)
         return full_response, [websocket]
 
-    def get_status(self):
+    def get_status(self) -> Dict:
         response = {"success": True, "genesis_initialized": True}
         return response
 
-    def plot_queue_to_payload(self, plot_queue_item):
+    def plot_queue_to_payload(self, plot_queue_item) -> Dict:
         error = plot_queue_item.get("error")
         has_error = error is not None
 
@@ -307,7 +309,7 @@ class WebSocketServer:
             "log": plot_queue_item.get("log"),
         }
 
-    def extract_plot_queue(self):
+    def extract_plot_queue(self) -> List[Dict]:
         data = []
         for item in self.plots_queue:
             data.append(WebSocketServer.plot_queue_to_payload(self, item))
@@ -355,13 +357,16 @@ class WebSocketServer:
         while True:
             new = await loop.run_in_executor(io_pool_exc, fp.readline)
 
+            if config["state"] is not PlotState.RUNNING:
+                return
+
             config["log"] = new if config["log"] is None else config["log"] + new
             self.state_changed(service_plotter, "log_changed")
 
             if new:
                 for word in words:
                     if word in new:
-                        yield (word, new)
+                        return
             else:
                 time.sleep(0.5)
 
@@ -371,8 +376,7 @@ class WebSocketServer:
         if config is None:
             raise Exception(f"Plot queue config with ID {id} is not defined")
 
-        async for hit_word, hit_sentence in self._watch_file_changes(id, loop):
-            break
+        await self._watch_file_changes(id, loop)
 
     def _build_plotting_command_args(self, request: Any, ignoreCount: bool) -> List[str]:
         service_name = request["service"]
@@ -455,6 +459,9 @@ class WebSocketServer:
             delay = config["delay"]
             await asyncio.sleep(delay)
 
+            if config["state"] is PlotState.DELETED:
+                return
+
             service_name = config["service_name"]
             command_args = config["command_args"]
             self.log.debug(f"command_args before launch_plotter are {command_args}")
@@ -538,7 +545,7 @@ class WebSocketServer:
 
         return response
 
-    async def stop_plotting(self, request: Dict[str, Any]):
+    async def stop_plotting(self, request: Dict[str, Any]) -> Dict:
         id = request["id"]
         config = self._get_plots_queue_item(id)
         if config is None:
@@ -549,6 +556,9 @@ class WebSocketServer:
         process = config["process"]
         queue = config["queue"]
 
+        # mark plot config as deleted for already running asyncio functions
+        config["state"] = PlotState.DELETED
+
         try:
             run_next = False
             if process is not None and state == PlotState.RUNNING:
@@ -567,7 +577,6 @@ class WebSocketServer:
             config["state"] = PlotState.ERROR
             config["error"] = str(e)
             self.state_changed(service_plotter, "state")
-            pass
             return {"success": False}
 
     async def start_service(self, request: Dict[str, Any]):
@@ -606,13 +615,13 @@ class WebSocketServer:
         response = {"success": success, "service": service_command, "error": error}
         return response
 
-    async def stop_service(self, request: Dict[str, Any]):
+    async def stop_service(self, request: Dict[str, Any]) -> Dict:
         service_name = request["service"]
         result = await kill_service(self.root_path, self.services, service_name)
         response = {"success": result, "service_name": service_name}
         return response
 
-    async def is_running(self, request: Dict[str, Any]):
+    async def is_running(self, request: Dict[str, Any]) -> Dict:
         service_name = request["service"]
 
         if service_name == service_plotter:
@@ -634,7 +643,7 @@ class WebSocketServer:
 
         return response
 
-    async def exit(self):
+    async def exit(self) -> Dict:
         jobs = []
         for k in self.services.keys():
             jobs.append(kill_service(self.root_path, self.services, k))
@@ -649,14 +658,14 @@ class WebSocketServer:
         response = {"success": True}
         return response
 
-    async def register_service(self, websocket: WebSocketServerProtocol, request: Dict[str, Any]):
+    async def register_service(self, websocket: WebSocketServerProtocol, request: Dict[str, Any]) -> Dict:
         self.log.info(f"Register service {request}")
         service = request["service"]
         if service not in self.connections:
             self.connections[service] = []
         self.connections[service].append(websocket)
 
-        response = {"success": False}
+        response: Dict[str, Any] = {"success": True}
         if service == service_plotter:
             response = {
                 "success": True,
@@ -667,7 +676,6 @@ class WebSocketServer:
             self.remote_address_map[websocket] = service
             if self.ping_job is None:
                 self.ping_job = asyncio.create_task(self.ping_task())
-            response = {"success": True}
         self.log.info(f"registered for service {service}")
         return response
 
