@@ -2,6 +2,7 @@ from pathlib import Path
 from typing import Dict, List
 
 from chia.rpc.rpc_client import RpcClient
+from chia.types.blockchain_format.coin import Coin
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.util.bech32m import decode_puzzle_hash
 from chia.util.ints import uint32, uint64
@@ -129,5 +130,18 @@ class WalletRpcClient(RpcClient):
     async def get_farmed_amount(self) -> Dict:
         return await self.fetch("get_farmed_amount", {})
 
+    async def create_signed_transaction(
+        self, additions: List[Dict], coins: List[Coin] = None, fee: uint64 = uint64(0)
+    ) -> Dict:
+        # Converts bytes to hex for puzzle hashes
+        additions_hex = [{"amount": ad["amount"], "puzzle_hash": ad["puzzle_hash"].hex()} for ad in additions]
+        if coins is not None and len(coins) > 0:
+            coins_json = [c.to_json_dict() for c in coins]
+            return await self.fetch(
+                "create_signed_transaction", {"additions": additions_hex, "coins": coins_json, "fee": fee}
+            )
+        else:
+            return await self.fetch("create_signed_transaction", {"additions": additions_hex, "fee": fee})
+
 
 # TODO: add APIs for coloured coins and RL wallet
