@@ -3,7 +3,6 @@ from typing import List
 
 from blspy import AugSchemeMPL, G2Element
 
-from chia.types.announcement import Announcement
 from chia.types.blockchain_format.coin import Coin
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.util.streamable import Streamable, streamable
@@ -40,12 +39,6 @@ class SpendBundle(Streamable):
             items.extend(coin_solution.additions())
         return items
 
-    def announcements(self) -> List[Announcement]:
-        items: List[Announcement] = []
-        for coin_solution in self.coin_solutions:
-            items.extend(coin_solution.announcements())
-        return items
-
     def removals(self) -> List[Coin]:
         """ This should be used only by wallet"""
         return [_.coin for _ in self.coin_solutions]
